/* SAMHAIN file system integrity testing                                   */
/* Copyright (C) 1999 Rainer Wichmann                                      */
/*                                                                         */
/*  This program is free software; you can redistribute it                 */
/*  and/or modify                                                          */
/*  it under the terms of the GNU General Public License as                */
/*  published by                                                           */
/*  the Free Software Foundation; either version 2 of the License, or      */
/*  (at your option) any later version.                                    */
/*                                                                         */
/*  This program is distributed in the hope that it will be useful,        */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*  GNU General Public License for more details.                           */
/*                                                                         */
/*  You should have received a copy of the GNU General Public License      */
/*  along with this program; if not, write to the Free Software            */
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              */

#include "config_xor.h"

#ifdef HAVE_BROKEN_INCLUDES
#define _ANSI_C_SOURCE
#define _POSIX_SOURCE
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#if TIME_WITH_SYS_TIME
#include <sys/time.h>
#include <time.h>
#else
#if HAVE_SYS_TIME_H
#include <sys/time.h>
#else
#include <time.h>
#endif
#endif


#ifdef SH_WITH_SERVER


#include "samhain.h"
#include "sh_forward.h"
#include "sh_error.h"
#include "sh_unix.h"
#include "sh_utils.h"
#include "sh_html.h"


s_stat  server_status;

static 
char outline[1024];

static 
char * replace_stat (char * line)
{
  int    i = 0;
  char * p = line;
  char * q;
  char   item[TIM_MAX];
  time_t    now;
  struct tm   * time_ptr;

  while (p && NULL != (q = strchr(p, '%')))
    {
      ++q;
      if (i > 0)
	*q = '%';
      else 
	{
	  switch (*q)
	    {
	    case 'T':
	      *q = 's'; 
	      now = time(NULL);
	      time_ptr   = localtime (&now);
	      if (time_ptr != NULL) 
		strftime (item, TIM_MAX-1, _("%d-%m-%Y %H:%M:%S"), time_ptr);
	      ++i;
	      break;
	    case 'S':
	      *q = 's'; 
	      time_ptr   = localtime (&(server_status.start));
	      if (time_ptr != NULL) 
		strftime (item, TIM_MAX-1, _("%d-%m-%Y %H:%M:%S"), time_ptr);
	      ++i;
	      break;
	    case 'L':
	      *q = 's'; 
	      time_ptr   = localtime (&(server_status.last));
	      if (time_ptr != NULL) 
		strftime (item, TIM_MAX-1, _("%d-%m-%Y %H:%M:%S"), time_ptr);
	      ++i;
	      break;
	      
	    case 'O':
	      *q = 's';
	      sl_snprintf(item, TIM_MAX-1, _("%d"),  server_status.conn_open);
	      ++i;
	      break;
	    case 'A':
	      *q = 's';
	      sl_snprintf(item, TIM_MAX-1, _("%ld"), server_status.conn_total);
	      ++i;
	      break;
	    case 'M':
	      *q = 's';
	      sl_snprintf(item, TIM_MAX-1, _("%d"),  server_status.conn_max);
	      ++i;
	      break;
	    default:
              *q = '%';
	      ++i;
              break;
	    }
	}
      ++q;
      p = q;
    }

  if      (i == 0)
    {
      return line;
    }
  else if (i == 1)
    {
       sl_snprintf(outline, 1023, line, item);
       return outline;
    }
  else
    return NULL;
}

static
int sh_html_head(SL_TICKET ticket)
{
  long      status = SL_ENONE;
  SL_TICKET fd = (-1);
  char      line[512];
  char      ts1[81];
  char      ts2[81];
  time_t    now;
  struct tm   * time_ptr;

  char    * formatted;

  char * p = sh_util_strconcat(DEFAULT_DATAROOT, "/head.html", NULL);

  if (p)
    {
      fd = sl_open_read (p, SL_YESPRIV);
      SH_FREE(p);
    }

  if (!SL_ISERROR(fd))
    {
      while (!SL_ISERROR(status) && sh_unix_getline (fd, line, 511) > 0) 
	{
	  formatted = replace_stat (line);
	  if (formatted)
	    status = sl_write_line (ticket, formatted, sl_strlen(formatted));
	}
      sl_close(fd);
    }
  else
    {
      status = 
	sl_write_line (ticket, 
		       _("<HTML><HEAD><TITLE>Report</TITLE></HEAD><BODY>"), 
		       sizeof("<HTML><HEAD><TITLE>Report</TITLE></HEAD><BODY>")-1); 
      if (!SL_ISERROR(status))
	status = 
	  sl_write_line (ticket, 
			 _("<H1>Samhain Server Report</H1>"), 
			 sizeof("<H1>Samhain Server Report</H1>")-1);
      if (!SL_ISERROR(status))
	{
	  time_ptr   = localtime (&(server_status.start));
	  if (time_ptr != NULL) 
	    status = strftime (ts1, 80, _("%d-%m-%Y %H:%M:%S"), time_ptr);
	  now = time(NULL);
	  time_ptr   = localtime (&now);
	  if (time_ptr != NULL) 
	    status = strftime (ts2, 80, _("%d-%m-%Y %H:%M:%S"), time_ptr);

	  sl_snprintf(outline, 1023, 
		      _("<p>Time:<BR>Now: %s<BR>Start: %s</p>"), 
		      ts2, ts1);
	  status = 
	    sl_write_line (ticket, outline, sl_strlen(outline));
	}
      if (!SL_ISERROR(status))
	{
	  sl_snprintf(outline, 1023, 
		      _("<p>Connections (max. %d simultaneous):"\
			"<BR>Now: %d<BR>Total: %ld</p>"),
		      server_status.conn_max,
		      server_status.conn_open,
		      server_status.conn_total);
	  status = 
	    sl_write_line (ticket, outline, sl_strlen(outline));
	  if (server_status.last > (time_t) 0)
	    {
	      time_ptr   = localtime (&(server_status.last));
	      if (time_ptr != NULL) 
		status = strftime (ts1, 80, _("%d-%m-%Y %H:%M:%S"), time_ptr);
	      sl_snprintf(outline, 1023, 
			  _("<p>Last connection at %s</p>"), 
			  ts1);
	      status = 
		sl_write_line (ticket, outline, sl_strlen(outline));
	    }
	}
      if (!SL_ISERROR(status))
	status = 
	  sl_write_line (ticket, 
			 _("<center><table cellpadding=5 cellspacing=2 border=2>"),
			 sizeof("<center><table cellpadding=5 cellspacing=2 border=2>")-1);
    }

  if (SL_ISERROR(status))
    return (-1);

  return(0);
}

static
int sh_html_foot(SL_TICKET ticket)
{
  long      status = SL_ENONE;
  SL_TICKET fd = (-1);
  char      line[512];

  char * p = sh_util_strconcat(DEFAULT_DATAROOT, "/foot.html", NULL);

  if (p)
    {
      fd = sl_open_read (p, SL_YESPRIV);
      SH_FREE(p);
    }

  if (!SL_ISERROR(fd))
    {
      while (!SL_ISERROR(status) && sh_unix_getline (fd, line, 511) > 0) 
	{
	  status = sl_write_line (ticket, line, sl_strlen(line));
	}
      sl_close(fd);
    }
  else
    {
      status =   sl_write_line (ticket, _("</table></center></BODY></HTML>"),
				sizeof("</table></center></BODY></HTML>")-1);
    }
  if (SL_ISERROR(status))
    return (-1);
  return(0);
}


static 
char * replace_tab (char * line, char * host, char * status, char * timestamp)
{
  int    i = 0;
  char * p = line;
  char * q;
  char * item[3];

  while (p && NULL != (q = strchr(p, '%')))
    {
      ++q;
      if      (*q == 'H' && i < 3)
	{
	  *q = 's'; item[i] = host; ++i;
	}
      else if (*q == 'S' && i < 3)
	{
	  *q = 's'; item[i] = status; ++i;
	}
      else if (*q == 'T' && i < 3)
	{
	  *q = 's'; item[i] = timestamp; ++i;
	}
      else 
	return NULL;

      ++q;
      p = q;
    }

  if      (i == 0)
    {
      return line;
    }
  else if (i == 1)
    {
       sl_snprintf(outline, 1023, line, item[0]);
       return outline;
    }
  else if (i == 2)
    {
       sl_snprintf(outline, 1023, line, item[0], item[1]);
       return outline;
    }
  else if (i == 3)
    {
       sl_snprintf(outline, 1023, line, item[0], item[1], item[2]); 
       return outline;
    }
  else
    return NULL;
      
  
}

static
int sh_html_entry (SL_TICKET ticket, 
		   char * host, char * status, char * timestamp)
{
  long      retval = SL_ENONE;
  SL_TICKET fd = (-1);
  char      line[512];

  char    * formatted;

  char * p = sh_util_strconcat(DEFAULT_DATAROOT, "/entry.html", NULL);

  if (p)
    {
      fd = sl_open_read (p, SL_YESPRIV);
      SH_FREE(p);
    }

  if (!SL_ISERROR(fd))
    {
      while (!SL_ISERROR(retval) && sh_unix_getline (fd, line, 511) > 0) 
	{
	  formatted = replace_tab(line, host, status, timestamp);
	  if (formatted)
	    retval = sl_write_line (ticket, formatted, sl_strlen(formatted));
	}
      sl_close(fd);
    }
  else
    {
      sl_snprintf(outline, 1023, 
		  _("<tr><td>%s</td><td>%s</td><td>%s</td></tr>"),
		  host, status, timestamp);
      retval =  sl_write_line (ticket, outline, sl_strlen(outline));
    }

  if (SL_ISERROR(retval))
    return (-1);
  return(0);
}

typedef struct _sort_arr {
  char msg[TIM_MAX];
  char tim[TIM_MAX];
} sort_arr;

static sort_arr sort_stat[CLT_MAX]; 

static
int comp_arr (const void * ao, const void * bo)
{
  sort_arr * a;
  sort_arr * b;

  if (ao == NULL && bo == NULL)
    return 0;
  else if (ao == NULL && bo != NULL)
    return (-1);
  else if (ao != NULL && bo == NULL)
    return (1);

  a = (sort_arr *) ao;
  b = (sort_arr *) bo;

  return ((-1) * sl_strcmp(a->tim, b->tim));
}

static
int sh_html_print_one (SL_TICKET ticket, client_t   * top)
{
  int status;
  int clt_status;
  int i, n;

  if (top == NULL)
    return 0;

  /* First, go to the left
   */
  status = sh_html_print_one (ticket, top->left);
  if (SL_ISERROR(status))
    return (-1);


  clt_status = top->status_now;
  status = sh_html_entry (ticket, 
			  top->hostname, 
			  _(clt_stat[clt_status]), 
			  top->timestamp[clt_status]);

  n = 0;

  if (clt_status != CLT_INACTIVE)
    {
      for (i = 1; i < CLT_MAX; ++i)
	{
	  if (top->status_arr[i] != CLT_INACTIVE)
	    {
	      clt_status = top->status_arr[i];
	      sl_strlcpy(sort_stat[n].msg, _(clt_stat[clt_status]),  TIM_MAX);
	      sl_strlcpy(sort_stat[n].tim, top->timestamp[clt_status],TIM_MAX);
	      ++n;
	    }
	}
    }

  if (n > 0)
    {
      qsort(&(sort_stat[0]), n, sizeof(sort_arr), comp_arr);
	  
      for (i = 1; i < n; ++i)
	{
	  status = sh_html_entry (ticket, 
				  "", 
				  sort_stat[i].msg,
				  sort_stat[i].tim);
	}
    }

  if (SL_ISERROR(status))
    return (-1);

  /* .. then go to the right.
   */
  status = sh_html_print_one (ticket, top->right);
  if (SL_ISERROR(status))
    return (-1);
  return 0;
}


int sh_html_write(void  * inptr)
{
  long fd;

  client_t * top = (client_t *) inptr;

  if (0 !=  tf_trust_check (DEFAULT_HTML_FILE, SL_YESPRIV))
    {
      sh_error_handle(SH_ERR_ERR,
		      _("Untrusted path: %s"),
		      DEFAULT_HTML_FILE);
      return (-1);
    } 


  fd = sl_open_write_trunc (DEFAULT_HTML_FILE, SL_YESPRIV);

  if (SL_ISERROR(fd))
    {
      sh_error_handle(SH_ERR_ERR,
		      _("Error opening %s"),
		      DEFAULT_HTML_FILE);
      return (-1);
    } 

  sh_html_head(fd);
  sh_html_print_one (fd, top);
  sh_html_foot(fd);
  sl_close(fd);

  return(0);
}

/* SH_WITH_SERVER */
#endif















